/*===========================================================================



Digital SWR meter
=================



Overview:
=========

CPU         :   PIC16F8777A or PIC16F874
Clock       :   XTAL 20.000MHz HS oscillator
Compiler    :   Hitech PICC
Version     :   1.00
Date        :   31.10.2008
Author      :   Iacopo Giangrandi HB9DUL
Adaptation  PCW compiler : F1TE


I/O connections:
================

See io.h for description.



Description:
============

...



============================================================================= */


#define 16F877 TRUE //uncomment for PIC16F874
// if 16F877 undefined -> 16F874 and no display power in dBm 

// --------------------------------------------------------------------------
// External files inclusions.
// --------------------------------------------------------------------------

#include "ROSMETRE.H"                                           // Use a PIC16F877A.
#include <math.h>
#include <lcd.c>                                                // LCD display handling routines.
#include <stdlib.h>
#include <stdio.h>


// --------------------------------------------------------------------------
// VFO definitions, constants and global variables.
// --------------------------------------------------------------------------
#define WELCOME_STRING_1    " Digital VSWR meter "
#define WELCOME_STRING_2    "v1.2 15.03.10 F1TE"
#define WELCOME_TIME        1000                        // 1 second.



    // Variables.
    
    signed int32 pwr_fwd=0;
    signed int32 pwr_rev=1;
    signed int32 a;
    float swr;
    int8 tempo;



// --------------------------------------------------------------------------
// This procedure converts the specified power into a 4 character ASCII
// string and prints it on the LCD screen. Power must be positive and smaller
// than 999. Leading zeros are removed. Decimal point is insered according
// to DP_0 and DP_1. The unit ("W") is added after the number.
// --------------------------------------------------------------------------
void write_power(signed int16 a)
{
    unsigned char ascii_num[5];                                 // Four characters string used to display the number in formatted ASCII decimal format.
    signed char i;                                              // Just a counter.

    if ((a > 999) || (a < 0))                                   // Make sure power is not too big.
    {
        lcd_putc("OVR ");                               // Write result on LCD screen.
        lcd_putc(" W");
        return;
    }

    if (input(DP_1) == 1)
    {
        if (input(DP_0) == 1)
        {
            for (i=3; i>=0; i--)                                // Convert into an ASCII string.
            {
                if (i == 1)                                     // Insert decimal point ("x.xx").
                    ascii_num[i] = '.';
                else
                {
                    ascii_num[i] = (a % 10) + '0';              // Get ASCII digit.
                    a /= 10;
                }
            }
        }
        else
        {
            for (i=3; i>=0; i--)                                // Convert into an ASCII string.
            {
                if (i == 2)                                     // Insert decimal point ("xx.x").
                    ascii_num[i] = '.';
                else
                {
                    ascii_num[i] = (a % 10) + '0';              // Get ASCII digit.
                    a /= 10;
                }
            }
            if (ascii_num[0] == '0')                            // Remove leading 0.
                ascii_num[0] = ' ';
        }
    }
    else
    {
        if (input(DP_0) == 1)
        {
            for (i=3; i>=0; i--)                                // Convert into an ASCII string.
            {
                if (i == 0)                                     // Insert space (no decimal point) (" xxx").
                    ascii_num[i] = ' ';
                else
                {
                    ascii_num[i] = (a % 10) + '0';              // Get ASCII digit.
                    a /= 10;
                }
            }
            for (i=1; i<3; i++)                                 // Remove leading zeros.
                if (ascii_num[i] == '0')                        // Replace 0 with spaces.
                    ascii_num[i] = ' ';
                else
                    break;                                      // Until the first non-zero digit is found.
        }
        else
        {
            for (i=3; i>=0; i--)                                // Convert into an ASCII string.
            {
                if (i == 3)                                     // Insert zero at the end (no decimal point) ("xxx0").
                    ascii_num[i] = '0';
                else
                {
                    ascii_num[i] = (a % 10) + '0';              // Get ASCII digit.
                    a /= 10;
                }
            }
            for (i=0; i<3; i++)                                 // Remove leading zeros.
                if (ascii_num[i] == '0')                        // Replace 0 with spaces.
                    ascii_num[i] = ' ';
                else
                    break;                                      // Until the first non-zero digit is found.
        }
    }    
    ascii_num[4] = 0x00;                                        // Null terminate the string.
    printf(lcd_putc,"%s",(ascii_num));                              // Write result on LCD screen.
    LCD_PUTC(" W");
}


// --------------------------------------------------------------------------
// This procedure converts the specified integer value into a 3 character
// ASCII string and prints it on the LCD screen. The value must be between
// -99 and +999. Leading zeros are removed and a minus sign is diplayed if
// needed. The unit (" dB") is added after the number.
// --------------------------------------------------------------------------
void write_db(signed int16 a)
{
    unsigned char ascii_num[4];                                 // Three characters string used to display the number in formatted ASCII decimal format.
    signed char i;                                              // Just a counter.
    unsigned char neg;

    if ((a < 0) || (a > 999))                                 // Make sure range is valid.
    {
        LCD_PUTC("---");
        LCD_PUTC(" dB");
        return;
    }

    neg = 0;
    if (a < 0)
    {
        neg = 1;
        a = -a;
    }

    for (i=2; i>=0; i--)                                        // Convert into an ASCII string.
    {
        ascii_num[i] = (a % 10) + '0';                          // Get ASCII digit.
        a /= 10;
    }

    for (i=0; i<2; i++)                                         // Remove leading zeros.
        if (ascii_num[i] == '0')                                // Replace 0 with spaces.
            ascii_num[i] = ' ';
        else
            break;                                              // Until the first non-zero digit is found.

    if (neg == 1)                                               // Place minus sign, if needed, right before the first digit.
    {
        if (ascii_num[1] == ' ')
            ascii_num[1] = '-';
        else
            ascii_num[0] = '-';
    }

    ascii_num[3] = 0x00;                                        // Null terminate the string.
    printf(lcd_putc,"%s",(ascii_num));                                // Write result on LCD screen.
    LCD_PUTC(" dB");
}



// --------------------------------------------------------------------------
// This procedure converts the specified SWR into a 10 character ASCII
// string containing a bar graph and prints it on the LCD screen.
// SWR is divided by 10.
// --------------------------------------------------------------------------
void write_swr_bar(float a)
{
//    unsigned char ascii_num[11];                                // Ten characters string used to display the number in formatted ASCII decimal format.
    signed char i,j; 
    lcd_gotoxy(10,2);
                                                                   // Just a counter and a temporary value.
    if (a < 0)                                                  // If SWR is infinite (represented as -1), make sure the bar is at its full length.
        a = 30000;

    if (a < 1.1)                                                 // Use a non-linear scale, first brick at 1.1:1.
        j = 0;
    else if (a < 1.2)                                            // Second brick at 1.2:1.
        j = 1;
    else if (a < 1.5)
        j = 2;
    else if (a < 1.8)
        j = 3;
    else if (a < 2.0)
        j = 4;
    else if (a < 2.5)
        j = 5;
    else if (a < 3.0)
        j = 6;
    else if (a < 4.0)
        j = 7;
    else if (a < 6.0)
        j = 8;
    else if (a < 8.0)
        j = 9;
    else
        j = 10;

    for (i=20; i>10; i--)                                        // Draw the bar-graph.
    {
        lcd_gotoxy(i,2);
        if (i > 20-j)
            lcd_putc(0xff);  
        else
            lcd_putc(" ");
    }

    // Write result on LCD screen.
}


#int_TIMER1
void TIMER1_isr(void)
{
tempo++;
}
// --------------------------------------------------------------------------
// This is the main program.
// --------------------------------------------------------------------------
void main(void)
{


   SET_TRIS_A(0b00001011);
   SET_TRIS_B(0b11110001);          // Port B en entre
   SET_TRIS_C(0b00000000);
//   SET_TRIS_E(0x07);
   
    setup_timer_1(T1_INTERNAL|T1_DIV_BY_8);  //104ms
  
//   setup_counters(RTCC_INTERNAL,RTCC_DIV_2);
//   setup_timer_1(T1_DISABLED);
  // setup_timer_2(T2_DIV_BY_16,255,16);
  // setup_ccp1(CCP_PWM);
  // setup_spi(FALSE);
   setup_adc_ports(RA0_RA1_RA3_ANALOG);
   setup_adc(ADC_CLOCK_INTERNAL);
   
   port_b_pullups(TRUE);
   disable_interrupts(INT_RDA);
   disable_interrupts(INT_EXT);
   disable_interrupts(INT_TIMER1);
   disable_interrupts(INT_TIMER2);
   enable_interrupts(global);


    
    // Initialization.
                                                 // Initialize I/O lines.
    LCD_init();                                                 // Initialize LCD display.



    // Welcome screen.

    output_bit(LCD_LIGHT,LIGHT_ON);                                       // Switch LCD light on.
    LCD_init();
    LCD_gotoxy(0,1);                                               // Display welcome screen.
    LCD_PUTC(WELCOME_STRING_1);
    LCD_gotoxy(0,2);
    LCD_PUTC(WELCOME_STRING_2);

    delay_ms(2000);                                        // Wait 1s
   output_bit(LCD_LIGHT,LIGHT_OFF);
 //  dp0=input(DP_0);
 //  dp1=input(DP_1);
//   scale0=input(SCALE_0);
//   scale1=input(SCALE_1);
    // Main loop.

    while (TRUE)                                                   // Endless loop tha polls PORTB waiting for user input.
    {
        if (input(SW_PEAK)==0)
           output_bit(PEAK,1);
        else
           output_bit(PEAK,0);
        set_adc_channel(0);
        delay_us(100);
        pwr_fwd = read_adc();
        delay_us(5);
        lcd_gotoxy(1,2);
        set_adc_channel(1);
        delay_us(100);                                             // Read volatges.
        pwr_rev = read_adc();
        delay_us(5);
        pwr_fwd *= pwr_fwd;                                     // Square to get power.
        pwr_rev *= pwr_rev;
        if ((input(SCALE_0) == 0) && (input(SCALE_1) == 1))
        {
            pwr_fwd *= 2;                                       // Full scale at 2W, 20W, 200W or 2kW.
            pwr_rev *= 2;
        }   
        else if ((input(SCALE_0) == 1) && (input(SCALE_1) == 0))
        {
            pwr_fwd *= 5;                                       // Full scale at 5W, 50W, 500W or 5kW.
            pwr_rev *= 5;
        }

        if (pwr_fwd != 0)                                       // Avoid dividing by 0.
        {
            swr = (float)pwr_rev / (float)pwr_fwd;              // Calculate SWR ( r=sqrt(P_r/P_f) ; SWR=(1+r)/(1-r) ).

            swr = sqrt(swr);

            swr = (1+swr)/(1-swr);

         if (swr>2) 
            output_bit(pin_c0,1);   // ALARME LED SI SWR > 2
         else 
            output_bit(pin_c0,0);

//         if (swr>3) 
//            output_bit(pin_c1,1);   // ALARME HARD SI SWR > 3
//         else 
            output_bit(pin_c1,0);
         
        }
        else                                                    // SWR is infinite.
            swr = -1;

        if ((pwr_fwd < 1000) && (pwr_rev < 1000))               // Switch LCD light on only when transmitting (power != 0).
        {
        /////////////////////////////////////////////////////////////// Tempo 1s sur allumage Backlight
          if (tempo>50)
           {
            tempo=0;
            disable_interrupts(INT_TIMER1);
            output_bit(LCD_LIGHT,LIGHT_OFF);
            }
        ///////////////////////////////////////////////////////////////////
            if (INPUT(SW_UNITS) == 1)                                  // Display W or dBm?
            {
                LCD_gotoxy(0,1);                                     // Display idle message for W.                
                LCD_PUTC("P fwd          P ref");
                LCD_gotoxy(0,2);
                LCD_PUTC("VSWR         <--VSWR");
            }
            else
            {
                LCD_gotoxy(0,1);                                     // Display idle message for dBm.
                
                LCD_PUTC("Lp fwd        Lp ref");
                LCD_gotoxy(0,2);
                LCD_PUTC("Return Loss  <--VSWR");
            }
        }
        else
        {
            output_bit(LCD_LIGHT,LIGHT_ON);
            enable_interrupts(INT_TIMER1); // Tempo 1s sur allumage Backlight
            if (INPUT(SW_UNITS) == 1)                                  // Display W or dBm?
            {
               LCD_gotoxy(0,1);                                     // Display W.
               write_power(pwr_fwd / 10000);
               LCD_PUTC("        ");                
               write_power(pwr_rev / 10000);
               LCD_PUTC(" W");
               LCD_gotoxy(0,2);                                   // Display SWR.
         if ((swr > 100) || (swr < 0))                                  // Make sure SWR is not infinite.
            LCD_PUTC("----");
         else    
             printf(lcd_putc,"%2.1f",(swr));                                // Write result on LCD screen.
               LCD_PUTC(":1      ");
               write_swr_bar(swr);
            }
            else
            {
#if defined 16F877 // no display in dBm for PIC16F874
               if (pwr_fwd != 0)                               // Avoid log(0) !!!
                {
                    a = 10 * log10(((float)pwr_fwd)/1000);     // Convert to dBm.
                    pwr_fwd = (int32)a;
                    if (input(DP_1) == 0)                              // Handle decimal point.
                        pwr_fwd += 20;
                    if (input(DP_0) == 0)
                        pwr_fwd += 10;
                }
                else
                    pwr_fwd = -10000;                           // -infinite (-10000dBm is small enough).

                if (pwr_rev != 0)                               // Avoid log(0) !!!
                {
                    a = 10 * log10(((float)pwr_rev)/1000);     // Convert to dBm.
                    pwr_rev = (int32)a;
                    if (input(DP_1) == 0)                              // Handle decimal point.
                        pwr_rev += 20;
                    if (input(DP_0) == 0)
                        pwr_rev += 10;
                }
                else
                    pwr_rev = -10000;                           // -infinite (-10000dBm is small enough).

                LCD_gotoxy(0,1);                                     // Display power in dBm.
                write_db(pwr_fwd);
                LCD_PUTC("m      ");
                write_db(pwr_rev);
                LCD_PUTC("m");
                LCD_gotoxy(0,2);                                   // Display return loss in dB and the usual SWR bar.
                write_db(pwr_fwd-pwr_rev);
                LCD_PUTC("    ");
                write_swr_bar(swr);
#endif                
            }
        }
    }
}


